package mvd.jester;

import java.util.HashSet;
import java.util.List;
import java.util.Set;
import mvd.jester.info.SchedulingInfo;
import mvd.jester.model.DagTask;
import mvd.jester.model.SortedTaskSet;
import mvd.jester.model.SynchronousTask;
import mvd.jester.model.Task;
import mvd.jester.model.SystemSetup.DagTaskBuilder;
import mvd.jester.model.SystemSetup.SynchronousTaskBuilder;
import mvd.jester.priority.PriorityManager;
import mvd.jester.tests.AbstractTest;

/**
 * TestEnvironment
 */
public class TestEnvironment {


    public TestEnvironment() {
    }

    public <T extends Task> Set<ResultCollector<AbstractTest<T>>> registerTests(
            List<AbstractTest<T>> tests) {
        Set<ResultCollector<AbstractTest<T>>> testSet = new HashSet<>();
        for (AbstractTest<T> t : tests) {
            testSet.add(new ResultCollector<AbstractTest<T>>(t.getPriorityManager(), t));
        }

        return testSet;
    }


    public void runExperiments(SynchronousTaskBuilder builder,
            Set<ResultCollector<AbstractTest<SynchronousTask>>> abstractTestInstances,
            long numberOfProcessors, long numberOfTaskSets) {

        long checkedTasksets = 0;
        System.out.print(Math.round((double) checkedTasksets / numberOfTaskSets * 100) + "% of "
                + numberOfTaskSets + " tasksets tested!\r");
        while (checkedTasksets < numberOfTaskSets) {
            Set<SynchronousTask> tasks = builder.generateTaskSet();

            double utilization = tasks.stream().mapToDouble(SynchronousTask::getUtilization).sum();

            while (utilization <= numberOfProcessors && checkedTasksets < numberOfTaskSets) {
                checkedTasksets++;

                if (checkedTasksets % 10 == 0) {
                    System.out.print(Math.round((double) checkedTasksets / numberOfTaskSets * 100)
                            + "% of " + numberOfTaskSets + " tasksets tested!\r");
                }

                for (ResultCollector<AbstractTest<SynchronousTask>> testInstance : abstractTestInstances) {
                    PriorityManager priorityManager = testInstance.getPriorityManager();
                    AbstractTest<SynchronousTask> abstractTest = testInstance.getAbstractValue();

                    SortedTaskSet<SynchronousTask> sortedTaskSet =
                            new SortedTaskSet<>(priorityManager);
                    sortedTaskSet.addAll(tasks);
                    SchedulingInfo schedulingInfo =
                            abstractTest.runSchedulabilityCheck(sortedTaskSet);
                    testInstance.addResult(schedulingInfo);
                }

                builder.addTask(tasks);

                utilization = tasks.stream().mapToDouble(SynchronousTask::getUtilization).sum();
            }
        }
        System.out.println("");
        ResultLogger resultLogger = new ResultLogger(numberOfProcessors);

        resultLogger.logTests(abstractTestInstances);
    }


    public void runExperiments(DagTaskBuilder builder,
            Set<ResultCollector<AbstractTest<DagTask>>> abstractTestInstances,
            long numberOfProcessors, long numberOfTaskSetsPerUtil) {

        long checkedTasksets = 0;
        long numberOfTaskSets = numberOfProcessors * 4 * numberOfTaskSetsPerUtil;

        // create LoggerFile here
        for (double util = 1; util <= numberOfProcessors; util += 0.25) {
            // new Result set here
            for (int i = 0; i < numberOfTaskSetsPerUtil; ++i) {
                Set<DagTask> taskSet = builder.generateTaskSet(util);

                System.out.print(Math.round((double) checkedTasksets / numberOfTaskSets * 100)
                        + "% of " + numberOfTaskSets + " tasksets tested!\r");
                for (ResultCollector<AbstractTest<DagTask>> testResultCollector : abstractTestInstances) {
                    AbstractTest<DagTask> testInstance = testResultCollector.getAbstractValue();
                    PriorityManager priorityManager = testResultCollector.getPriorityManager();

                    SortedTaskSet<DagTask> sortedTaskSet = new SortedTaskSet<>(priorityManager);
                    sortedTaskSet.addAll(taskSet);

                    SchedulingInfo schedulingInfo =
                            testInstance.runSchedulabilityCheck(sortedTaskSet);

                    testResultCollector.addResult(schedulingInfo);
                }
                checkedTasksets++;
            }
            // write Result line to LoggerFile
        }
        System.out.println("");
        ResultLogger resultLogger = new ResultLogger(numberOfProcessors);

        resultLogger.logTests(abstractTestInstances);
    }
}


